@@ -41,7 +41,8 @@ from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
 from neutron.cmd import runtime_checks as checks
-from neutron.common import _constants as common_constants
+from neutron.common.ovn import constants as ovn_constants
+from neutron.common.ovn import utils as ovn_utils
 from neutron.common import utils as common_utils
 from neutron.ipam import utils as ipam_utils
 from neutron.privileged.agent.linux import dhcp as priv_dhcp
@@ -295,7 +296,7 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
         """Returns the file name for a given kind of config file."""
         return os.path.join(self.network_conf_dir, kind)
 
-    def get_process_uuid(self):
+    def _get_process_uuid(self):
         if self.segment:
             # NOTE(sahid): Keep the order to match directory path. This is used
             # by external_process.ProcessManager to check whether the process
@@ -348,7 +349,7 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
     def _get_process_manager(self, cmd_callback=None):
         return external_process.ProcessManager(
             conf=self.conf,
-            uuid=self.get_process_uuid(),
+            uuid=self._get_process_uuid(),
             namespace=self.network.namespace,
             service=DNSMASQ_SERVICE_NAME,
             default_cmd_callback=cmd_callback,
@@ -1213,7 +1214,8 @@ class Dnsmasq(DhcpLocalProcess):
         ports_result = self.device_manager.plugin.get_ports(
             port_filters={
                 'device_owner': [constants.DEVICE_OWNER_DISTRIBUTED],
-                'device_id': ['ovnmeta-' + self.network.id]
+                'device_id':
+                    [ovn_constants.OVN_METADATA_PREFIX + self.network.id]
             },
         )
         if ports_result:
@@ -1465,11 +1467,6 @@ class Dnsmasq(DhcpLocalProcess):
             return True
         return False
 
-    @staticmethod
-    def _is_ovn_metadata_port(port, network_id):
-        return (port.device_id == 'ovnmeta-' + network_id and
-                port.device_owner == constants.DEVICE_OWNER_DISTRIBUTED)
-
     @classmethod
     def should_enable_metadata(cls, conf, network):
         """Determine whether the metadata proxy is needed for a network
@@ -1489,7 +1486,7 @@ class Dnsmasq(DhcpLocalProcess):
         with 3rd party backends.
         """
         for port in network.ports:
-            if cls._is_ovn_metadata_port(port, network.id):
+            if ovn_utils.is_ovn_metadata_port(port):
                 return False
 
         all_subnets = cls._get_all_subnets(network)
@@ -1768,8 +1765,7 @@ class DeviceManager(object):
                      # to us (e.g. auto ipv6 addresses)
                      if fixed_ip.subnet_id in dhcp_subnets]
 
-        ips = [DictModel(item) if isinstance(item, dict) else item
-               for item in fixed_ips]
+        ips = [DictModel(item) for item in fixed_ips]
         dhcp_port.fixed_ips = ips
 
         return dhcp_port
@@ -1784,7 +1780,7 @@ class DeviceManager(object):
 
     def cleanup_stale_devices(self, network, dhcp_port):
         """Unplug unrelated or stale devices found in the namespace."""
-        LOG.debug("Cleaning stale devices for network %s", network.id)
+        LOG.info("Cleaning stale devices for network %s", network.id)
         skip_dev_name = (self.driver.get_device_name(dhcp_port)
                          if dhcp_port else None)
         ns_ip = ip_lib.IPWrapper(namespace=network.namespace)
@@ -1797,7 +1793,7 @@ class DeviceManager(object):
 
         for dev_name in hw_ports - db_ports:
             if dev_name != skip_dev_name:
-                LOG.debug("Found stale device %s, deleting", dev_name)
+                LOG.warning("Found stale device %s, deleting", dev_name)
                 try:
                     self.unplug(dev_name, network)
                 except Exception:
@@ -1869,12 +1865,16 @@ class DeviceManager(object):
             ip_cidr = '%s/%s' % (fixed_ip.ip_address, net.prefixlen)
             ip_cidrs.append(ip_cidr)
 
-        if self.driver.use_gateway_ips:
+        need_ipv6_metadata = False
+        for subnet in network.subnets:
+            if subnet.ip_version == constants.IP_VERSION_6:
+                need_ipv6_metadata = True
+
+            if not self.driver.use_gateway_ips:
+                continue
             # For each DHCP-enabled subnet, add that subnet's gateway
             # IP address to the Linux device for the DHCP port.
-            for subnet in network.subnets:
-                if not subnet.enable_dhcp:
-                    continue
+            if subnet.enable_dhcp:
                 gateway = subnet.gateway_ip
                 if gateway:
                     net = netaddr.IPNetwork(subnet.cidr)
@@ -1882,8 +1882,8 @@ class DeviceManager(object):
 
         if self.conf.force_metadata or self.conf.enable_isolated_metadata:
             ip_cidrs.append(constants.METADATA_CIDR)
-            if netutils.is_ipv6_enabled():
-                ip_cidrs.append(common_constants.METADATA_V6_CIDR)
+            if need_ipv6_metadata and netutils.is_ipv6_enabled():
+                ip_cidrs.append(constants.METADATA_V6_CIDR)
 
         self.driver.init_l3(interface_name, ip_cidrs,
                             namespace=network.namespace)
