@@ -19,6 +19,7 @@ from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
 from neutron_lib.utils import helpers
 from oslo_log import log
+from oslo_utils import strutils
 from oslo_utils import uuidutils
 from ovs import socket_util
 from ovs import stream
@@ -300,14 +301,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             raise RuntimeError(_("Currently only supports "
                                  "delete by lport-name"))
 
-    def get_all_stateless_fip_nats(self):
-        cmd = self.db_find(
-            'NAT',
-            ('external_ids', '!=', {ovn_const.OVN_FIP_EXT_ID_KEY: ''}),
-            ('options', '=', {'stateless': 'true'}),
-            ('type', '=', 'dnat_and_snat'))
-        return cmd.execute(check_error=True)
-
     def get_all_logical_switches_with_ports(self):
         result = []
         for lswitch in self._tables['Logical_Switch'].rows.values():
@@ -431,21 +424,26 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
                 acl_list.append(acl_string)
         return acl_values_dict, acl_obj_dict, lswitch_ovsdb_dict
 
-    def create_lrouter(self, name, may_exist=True, **columns):
-        return cmd.AddLRouterCommand(self, name,
-                                     may_exist, **columns)
-
     def update_lrouter(self, name, if_exists=True, **columns):
         return cmd.UpdateLRouterCommand(self, name,
                                         if_exists, **columns)
 
-    def delete_lrouter(self, name, if_exists=True):
-        return cmd.DelLRouterCommand(self, name, if_exists)
-
     def add_lrouter_port(self, name, lrouter, may_exist=False, **columns):
         return cmd.AddLRouterPortCommand(self, name, lrouter,
                                          may_exist, **columns)
 
+    def schedule_unhosted_gateways(self, g_name, sb_api, plugin, port_physnets,
+                                   all_gw_chassis, chassis_with_physnets,
+                                   chassis_with_azs):
+        return cmd.ScheduleUnhostedGatewaysCommand(
+            self, g_name, sb_api, plugin, port_physnets, all_gw_chassis,
+            chassis_with_physnets, chassis_with_azs)
+
+    def schedule_new_gateway(self, g_name, sb_api, lrouter_name, plugin,
+                             physnet, az_hints):
+        return cmd.ScheduleNewGatewayCommand(
+            self, g_name, sb_api, lrouter_name, plugin, physnet, az_hints)
+
     def update_lrouter_port(self, name, if_exists=True, **columns):
         return cmd.UpdateLRouterPortCommand(self, name, if_exists, **columns)
 
@@ -467,29 +465,33 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def delete_acl(self, lswitch, lport, if_exists=True):
         return cmd.DelACLCommand(self, lswitch, lport, if_exists)
 
-    def add_static_route(self, lrouter, **columns):
-        return cmd.AddStaticRouteCommand(self, lrouter, **columns)
+    def add_static_route(self, lrouter, maintain_bfd=False, **columns):
+        return cmd.AddStaticRouteCommand(self, lrouter, maintain_bfd,
+                                         **columns)
 
     def delete_static_route(self, lrouter, ip_prefix, nexthop, if_exists=True):
         return cmd.DelStaticRouteCommand(self, lrouter, ip_prefix, nexthop,
                                          if_exists)
 
-    def delete_address_set(self, name, if_exists=True, **columns):
-        return cmd.DelAddrSetCommand(self, name, if_exists)
-
-    def _get_logical_router_port_gateway_chassis(self, lrp):
+    def _get_logical_router_port_gateway_chassis(self, lrp, priorities=None):
         """Get the list of chassis hosting this gateway port.
 
         @param   lrp: logical router port
         @type    lrp: Logical_Router_Port row
-        @return: List of tuples (chassis_name, priority) sorted by priority
+        @param   priorities: a list of gateway chassis priorities to search for
+        @type    priorities: list of int
+        @return: List of tuples (chassis_name, priority) sorted by priority. If
+                 ``priorities`` is set then only chassis matching of of these
+                 priorities are returned.
         """
         # Try retrieving gateway_chassis with new schema. If new schema is not
         # supported or user is using old schema, then use old schema for
         # getting gateway_chassis
         chassis = []
         if self._tables.get('Gateway_Chassis'):
-            for gwc in lrp.gateway_chassis:
+            for gwc in getattr(lrp, 'gateway_chassis', set()):
+                if priorities is not None and gwc.priority not in priorities:
+                    continue
                 chassis.append((gwc.chassis_name, gwc.priority))
         else:
             rc = lrp.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
@@ -499,14 +501,16 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return sorted(chassis, reverse=True, key=lambda x: x[1])
 
     def get_all_chassis_gateway_bindings(self,
-                                         chassis_candidate_list=None):
+                                         chassis_candidate_list=None,
+                                         priorities=None):
         chassis_bindings = {}
         for chassis_name in chassis_candidate_list or []:
             chassis_bindings.setdefault(chassis_name, [])
         for lrp in self._tables['Logical_Router_Port'].rows.values():
             if not lrp.name.startswith('lrp-'):
                 continue
-            chassis = self._get_logical_router_port_gateway_chassis(lrp)
+            chassis = self._get_logical_router_port_gateway_chassis(
+                lrp, priorities=priorities)
             for chassis_name, prio in chassis:
                 if (not chassis_candidate_list or
                         chassis_name in chassis_candidate_list):
@@ -548,8 +552,16 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
     def get_unhosted_gateways(self, port_physnet_dict, chassis_with_physnets,
                               all_gw_chassis, chassis_with_azs):
+        """Return the GW LRPs with no chassis assigned
+
+        If the LRP belongs to a tunnelled network (physnet=None), it won't be
+        hosted to any chassis.
+        """
         unhosted_gateways = set()
         for port, physnet in port_physnet_dict.items():
+            if not physnet:
+                continue
+
             lrp_name = '%s%s' % (ovn_const.LRP_PREFIX, port)
             original_state = self.get_gateway_chassis_binding(lrp_name)
             az_hints = self.get_gateway_chassis_az_hints(lrp_name)
@@ -643,18 +655,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
         return dhcp_options
 
-    def get_address_sets(self):
-        address_sets = {}
-        for row in self._tables['Address_Set'].rows.values():
-            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids):
-                continue
-            name = getattr(row, 'name')
-            data = {}
-            for row_key in getattr(row, "_data", {}):
-                data[row_key] = getattr(row, row_key)
-            address_sets[name] = data
-        return address_sets
-
     def get_router_port_options(self, lsp_name):
         try:
             lsp = idlutils.row_by_value(self.idl, 'Logical_Switch_Port',
@@ -662,7 +662,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             options = getattr(lsp, 'options')
             for key in list(options.keys()):
                 if key not in ovn_const.OVN_ROUTER_PORT_OPTION_KEYS:
-                    del(options[key])
+                    del options[key]
             return options
         except idlutils.RowNotFound:
             return {}
@@ -762,16 +762,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         result = fip.execute(check_error=True)
         return result[0] if result else None
 
-    def get_floatingip_by_ips(self, router_id, logical_ip, external_ip):
-        if not all([router_id, logical_ip, external_ip]):
-            return
-
-        for nat in self.get_lrouter_nat_rules(utils.ovn_name(router_id)):
-            if (nat['type'] == 'dnat_and_snat' and
-                    nat['logical_ip'] == logical_ip and
-                    nat['external_ip'] == external_ip):
-                return nat
-
     def check_revision_number(self, name, resource, resource_type,
                               if_exists=True):
         return cmd.CheckRevisionNumberCommand(
@@ -795,29 +785,63 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return result[0] if result else None
 
     def get_lrouter_gw_ports(self, lrouter_name):
+        r_name = ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY
+        is_gw = ovn_const.OVN_ROUTER_IS_EXT_GW
         lr = self.get_lrouter(lrouter_name)
         gw_ports = []
         for lrp in getattr(lr, 'ports', []):
             lrp_ext_ids = getattr(lrp, 'external_ids', {})
-            if (ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY not in lrp_ext_ids or
-                    utils.ovn_name(lrp_ext_ids[
-                        ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY]) != lr.name):
+            if (r_name not in lrp_ext_ids or
+                    utils.ovn_name(lrp_ext_ids[r_name]) != lr.name or
+                    not strutils.bool_from_string(lrp_ext_ids.get(is_gw))):
                 continue
-            lrp_ha_cfg = (getattr(lrp, 'gateway_chassis', None) or
-                          getattr(lrp, 'options', {}).get(
-                              ovn_const.OVN_GATEWAY_CHASSIS_KEY))
-            if lrp_ha_cfg:
-                gw_ports.append(lrp)
+
+            gw_ports.append(lrp)
         return gw_ports
 
-    def delete_lrouter_ext_gw(self, lrouter_name, if_exists=True):
-        return cmd.DeleteLRouterExtGwCommand(self, lrouter_name, if_exists)
+    def get_lrouter_by_lrouter_port(self, lrp_name):
+        """Get LR by name of LRP.
+
+        :param lrp_name: Name of LRP.
+        :type lrp_name:  str
+        :returns:        LR associated with LRP as represented by lrp_name.
+        :rtype:          Optional[ovs_idl.rowview.RowView]
+        """
+        lrp = self.get_lrouter_port(lrp_name)
+        if not lrp:
+            return None
+
+        # NOTE(fnordahl) This could be replaced by something like:
+        #
+        #     lr = self.db_find_rows(
+        #         'Logical_Router',
+        #         ('ports', '{>}', lrp.uuid))
+        #
+        # However, ovsdbapp does not currently support the '{>}' operator.
+        for lr in self._tables['Logical_Router'].rows.values():
+            lr_ports = getattr(lr, 'ports', set())
+            if lrp in lr_ports:
+                return lr
+        return None
+
+    def delete_lrouter_ext_gw(self, lrouter_name, if_exists=True,
+                              maintain_bfd=True):
+        return cmd.DeleteLRouterExtGwCommand(self, lrouter_name, if_exists,
+                                             maintain_bfd)
 
     def get_port_group(self, pg_name):
         if uuidutils.is_uuid_like(pg_name):
             pg_name = utils.ovn_port_group_name(pg_name)
         return self.lookup('Port_Group', pg_name, default=None)
 
+    def get_address_set(self, as_name):
+        if uuidutils.is_uuid_like(as_name):
+            as_name_v4 = utils.ovn_ag_addrset_name(as_name, 'ip4')
+            as_name_v6 = utils.ovn_ag_addrset_name(as_name, 'ip6')
+            return (self.lookup('Address_Set', as_name_v4, default=None),
+                    self.lookup('Address_Set', as_name_v6, default=None))
+        return self.lookup('Address_Set', as_name, default=None), None
+
     def get_sg_port_groups(self):
         """Returns OVN port groups used as Neutron Security Groups.
 
@@ -853,6 +877,15 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def update_lb_external_ids(self, lb_name, values, if_exists=True):
         return cmd.UpdateLbExternalIds(self, lb_name, values, if_exists)
 
+    def set_nb_global_options(self, **options):
+        LOG.debug("Setting NB_Global options: %s", options)
+        return self.db_set("NB_Global", ".", options=options)
+
+    def set_router_mac_age_limit(self, router=None):
+        # Set the MAC_Binding age limit on OVN Logical Routers
+        return cmd.SetLRouterMacAgeLimitCommand(
+            self, router, cfg.get_ovn_mac_binding_age_threshold())
+
 
 class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def __init__(self, connection):
@@ -950,7 +983,7 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         return cmd.UpdateChassisExtIdsCommand(
             self, chassis, {desc_key: description}, if_exists=False)
 
-    def get_network_port_bindings_by_ip(self, network, ip_address):
+    def get_network_port_bindings_by_ip(self, network, ip_address, mac=None):
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
         # TODO(twilson) It would be useful to have a db_find that takes a
         # comparison function
@@ -959,12 +992,23 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             # If the port is not bound to any chassis it is not relevant
             if not port.chassis:
                 return False
+            if not port.mac:
+                return False
+            # The MAC and IP address(es) are both present in port.mac as
+            # ["MAC IP {IP2...IPN}"]. If either one is present that is a
+            # match, since for link-local clients we can only match the MAC.
+            mac_ip = port.mac[0].split(' ')
+            address_match = False
+            if mac and mac in mac_ip:
+                address_match = True
+            elif ip_address in mac_ip:
+                address_match = True
+            if not address_match:
+                return False
 
             is_in_network = utils.get_network_name_from_datapath(
                 port.datapath) == network
-            return (port.mac and
-                    is_in_network and
-                    (ip_address in port.mac[0].split(' ')))
+            return is_in_network
 
         return [r for r in rows if check_net_and_ip(r)]
 
@@ -973,11 +1017,18 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         return self.db_set('Port_Binding', name, 'external_ids',
                            {'neutron-port-cidrs': cidrs})
 
-    def get_ports_on_chassis(self, chassis):
+    def get_ports_on_chassis(self, chassis, include_additional_chassis=False):
         # TODO(twilson) Some day it would be nice to stop passing names around
         # and just start using chassis objects so db_find_rows could be used
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
-        return [r for r in rows if r.chassis and r.chassis[0].name == chassis]
+        if (include_additional_chassis and
+                utils.is_additional_chassis_supported(self)):
+            return [r for r in rows
+                    if r.chassis and r.chassis[0].name == chassis or
+                    chassis in [ch.name for ch in r.additional_chassis]]
+        else:
+            return [r for r in rows
+                    if r.chassis and r.chassis[0].name == chassis]
 
     def get_chassis_host_for_port(self, port_id):
         chassis = set()
