@@ -14,6 +14,7 @@
 #    under the License.
 
 from collections import abc
+import copy
 import itertools
 import re
 import sys
@@ -26,7 +27,6 @@ from neutron_lib import exceptions
 from neutron_lib.plugins import directory
 from neutron_lib.services import constants as service_const
 from oslo_config import cfg
-from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_policy import opts
 from oslo_policy import policy
@@ -51,12 +51,15 @@ _RESOURCE_FOREIGN_KEYS = {
     'security_groups': 'security_group_id'
 }
 
-
-# TODO(gmann): Remove setting the default value of config policy_file
-# once oslo_policy change the default value to 'policy.yaml'.
-# https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+# TODO(slaweq): Remove overriding the default value of config options
+# 'policy_file', 'enforce_scope', and 'enforce_new_defaults' once
+# oslo_policy change their default value to what is overridden here.
 DEFAULT_POLICY_FILE = 'policy.yaml'
-opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
+opts.set_defaults(
+    cfg.CONF,
+    DEFAULT_POLICY_FILE,
+    enforce_scope=True,
+    enforce_new_defaults=True)
 
 
 def reset():
@@ -143,8 +146,8 @@ def _should_validate_sub_attributes(attribute, sub_attr):
     """Verify that sub-attributes are iterable and should be validated."""
     validate = attribute.get('validate')
     return (validate and isinstance(sub_attr, abc.Iterable) and
-            any([k.startswith('type:dict') and
-                 v for (k, v) in validate.items()]))
+            any(k.startswith('type:dict') and v
+                for (k, v) in validate.items()))
 
 
 def _build_subattr_match_rule(attr_name, attr, action, target):
@@ -287,7 +290,8 @@ class OwnerCheck(policy.Check):
         return OwnerCheck(self._orig_kind, self._orig_match)
 
     @cache.cache_method_results
-    def _extract(self, resource_type, resource_id, field):
+    def _extract(self, resource_type, resource_id, field,
+                 retry_if_not_found=True):
         # NOTE(salv-orlando): This check currently assumes the parent
         # resource is handled by the core plugin. It might be worth
         # having a way to map resources to plugins so to make this
@@ -304,18 +308,22 @@ class OwnerCheck(policy.Check):
                      resource_id,
                      fields=[field])
         except exceptions.NotFound as e:
-            # NOTE(kevinbenton): a NotFound exception can occur if a
+            # NOTE(kevinbenton, slaweq): a NotFound exception can occur if a
             # list operation is happening at the same time as one of
             # the parents and its children being deleted. So we issue
-            # a RetryRequest so the API will redo the lookup and the
+            # retry to get it once again if we didn't yet.
             # problem items will be gone.
-            raise db_exc.RetryRequest(e)
+            if retry_if_not_found:
+                return self._extract(resource_type, resource_id, field,
+                                     retry_if_not_found=False)
+            raise e
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception('Policy check error while calling %s!', f)
         return data[field]
 
     def __call__(self, target, creds, enforcer):
+        target_copy = copy.copy(target)
         if self.target_field not in target:
             # policy needs a plugin check
             # target field is in the form resource:field
@@ -360,10 +368,10 @@ class OwnerCheck(policy.Check):
                     policy="%s:%s" % (self.kind, self.match),
                     reason=err_reason)
 
-            target[self.target_field] = self._extract(
+            target_copy[self.target_field] = self._extract(
                 parent_res, target[parent_foreign_key], parent_field)
 
-        match = self.match % target
+        match = self.match % target_copy
         if self.kind in creds:
             return match == str(creds[self.kind])
         return False
@@ -383,11 +391,15 @@ class FieldCheck(policy.Check):
                                          (resource, field, value))
 
         # Value might need conversion - we need help from the attribute map
+
+        def _no_conv(x):
+            return x
+
         try:
             attr = attributes.RESOURCES[resource][field]
             conv_func = attr['convert_to']
         except KeyError:
-            conv_func = lambda x: x
+            conv_func = _no_conv
 
         self.field = field
         self.resource = resource
@@ -477,6 +489,7 @@ def check(context, action, target, plugin=None, might_not_exist=False,
     # personas will be supported
     if not cfg.CONF.oslo_policy.enforce_new_defaults and context.is_admin:
         return True
+    init()
     if might_not_exist and not (_ENFORCER.rules and action in _ENFORCER.rules):
         return True
     match_rule, target, credentials = _prepare_check(context,
@@ -513,6 +526,7 @@ def enforce(context, action, target, plugin=None, pluralized=None):
     # personas will be supported
     if not cfg.CONF.oslo_policy.enforce_new_defaults and context.is_admin:
         return True
+    init()
     rule, target, context = _prepare_check(context, action, target, pluralized)
     try:
         result = _ENFORCER.enforce(rule, target, context, action=action,
