@@ -22,7 +22,6 @@ from oslo_config import cfg
 from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.agent.common import ovs_lib
-from neutron.agent.linux import ip_lib
 from neutron.common import utils
 from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import base
@@ -205,8 +204,10 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
         br_other_config = self.ovs.ovsdb.db_find(
             'Bridge', ('name', '=', self.br.br_name), columns=['other_config']
         ).execute()[0]['other_config']
+        expected_flood_value = ('false' if
+            cfg.CONF.OVS.igmp_flood_unregistered else 'true')
         self.assertEqual(
-            'false',
+            expected_flood_value,
             br_other_config.get(
                 'mcast-snooping-disable-flood-unregistered', '').lower())
 
@@ -216,13 +217,6 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
     def test_set_igmp_snooping_disabled(self):
         self._test_set_igmp_snooping_state(False)
 
-    def test_get_datapath_id(self):
-        brdev = ip_lib.IPDevice(self.br.br_name)
-        dpid = brdev.link.attributes['link/ether'].replace(':', '')
-        self.br.set_db_attribute('Bridge',
-                                 self.br.br_name, 'datapath_id', dpid)
-        self.assertIn(dpid, self.br.get_datapath_id())
-
     def _test_add_tunnel_port(self, attrs,
                               expected_tunnel_type=const.TYPE_GRE):
         port_name = utils.get_rand_device_name(net_helpers.PORT_PREFIX)
@@ -307,7 +301,7 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
         vif_ports = [self.create_ovs_vif_port() for i in range(3)]
         ports = self.br.get_vif_ports()
         self.assertEqual(3, len(ports))
-        self.assertTrue(all([isinstance(x, ovs_lib.VifPort) for x in ports]))
+        self.assertTrue(all(isinstance(x, ovs_lib.VifPort) for x in ports))
         self.assertEqual(sorted([x.port_name for x in vif_ports]),
                          sorted([x.port_name for x in ports]))
 
@@ -323,7 +317,7 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
                           new=new_port_name_list).start()
         ports = self.br.get_vif_ports()
         self.assertEqual(3, len(ports))
-        self.assertTrue(all([isinstance(x, ovs_lib.VifPort) for x in ports]))
+        self.assertTrue(all(isinstance(x, ovs_lib.VifPort) for x in ports))
         self.assertEqual(sorted([x.port_name for x in vif_ports]),
                          sorted([x.port_name for x in ports]))
 
@@ -485,7 +479,7 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
                 txn.add(ovsdb.del_port(port_name, self.br.br_name,
                                        if_exists=False))
                 txn.add(ovsdb.db_set('Interface', port_name,
-                                     ('type', 'internal')))
+                                     ('type', 'internal'), if_exists=False))
         self.assertRaises((RuntimeError, idlutils.RowNotFound),
                           del_port_mod_iface)
 
