@@ -17,21 +17,25 @@ from unittest import mock
 
 from oslo_config import cfg
 from oslo_utils import strutils
-from oslo_utils import uuidutils
 
 from futurist import periodics
 from neutron_lib.api.definitions import external_net as extnet_apidef
 from neutron_lib.api.definitions import floating_ip_port_forwarding as pf_def
 from neutron_lib.api.definitions import provider_net as provnet_apidef
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
 from neutron_lib import constants as n_const
 from neutron_lib import context as n_context
 from neutron_lib.exceptions import l3 as lib_l3_exc
+from oslo_utils import uuidutils
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as ovn_config
 from neutron.db import ovn_revision_numbers_db as db_rev
+from neutron.objects import servicetype as servicetype_obj
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import maintenance
+from neutron.services.portforwarding import constants as pf_consts
 from neutron.tests.functional import base
 from neutron.tests.functional.services.logapi.drivers.ovn \
     import test_driver as test_log_driver
@@ -177,9 +181,9 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['router']
 
-    def _update_router_name(self, net_id, new_name):
-        data = {'router': {'name': new_name}}
-        req = self.new_update_request('routers', data, net_id, self.fmt)
+    def _update_router(self, router_id, router_dict):
+        data = {'router': router_dict}
+        req = self.new_update_request('routers', data, router_id, self.fmt)
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['router']
 
@@ -594,8 +598,8 @@ class TestMaintenance(_TestMaintenanceHelper):
 
         new_obj_name = 'routertest_updated'
         with mock.patch.object(self._l3_ovn_client, 'update_router'):
-            new_neutron_obj = self._update_router_name(neutron_obj['id'],
-                                                       new_obj_name)
+            new_neutron_obj = self._update_router(neutron_obj['id'],
+                                                  {'name': new_obj_name})
 
         # Assert the revision numbers are out-of-sync
         ovn_obj = self._find_router_row_by_name(obj_name)
@@ -771,6 +775,75 @@ class TestMaintenance(_TestMaintenanceHelper):
         self.assertEqual(
             'false', ls['other_config'][ovn_const.MCAST_FLOOD_UNREGISTERED])
 
+    def test_check_for_aging_settings(self):
+        net = self._create_network('net', provider='datacentre')
+        ls = self.nb_api.get_lswitch(utils.ovn_name(net['id']))
+
+        self.assertEqual(
+            '0', ls.other_config.get(ovn_const.LS_OPTIONS_FDB_AGE_THRESHOLD))
+
+        self.assertEqual(
+            '0', self.nb_api.nb_global.options.get("fdb_removal_limit", '0'))
+
+        # Change the value of the configuration
+        cfg.CONF.set_override('fdb_age_threshold', 5, group='ovn')
+        cfg.CONF.set_override('fdb_removal_limit', 100, group='ovn_nb_global')
+
+        # Call the maintenance task and check that the value has been
+        # updated in the Logical Switch
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_fdb_aging_settings)
+
+        ls = self.nb_api.get_lswitch(utils.ovn_name(net['id']))
+
+        self.assertEqual(
+            '5', ls.other_config.get(ovn_const.LS_OPTIONS_FDB_AGE_THRESHOLD))
+        self.assertEqual(
+            '100', self.nb_api.nb_global.options.get("fdb_removal_limit"))
+
+    def test_update_mac_aging_settings(self):
+        ext_net = self._create_network('ext_networktest', external=True)
+        ext_subnet = self._create_subnet(
+            'ext_subnettest',
+            ext_net['id'],
+            **{'cidr': '100.0.0.0/24',
+               'gateway_ip': '100.0.0.254',
+               'allocation_pools': [
+                   {'start': '100.0.0.2', 'end': '100.0.0.253'}],
+               'enable_dhcp': False})
+        self._create_network('network1test', external=False)
+        external_gateway_info = {
+            'enable_snat': True,
+            'network_id': ext_net['id'],
+            'external_fixed_ips': [
+                {'ip_address': '100.0.0.2', 'subnet_id': ext_subnet['id']}]}
+        router = self._create_router(
+            'routertest', external_gateway_info=external_gateway_info)
+
+        options = self.nb_api.nb_global.options
+        lr = self.nb_api.get_lrouter(router["id"])
+
+        self.assertEqual(
+            '0', lr.options.get(ovn_const.LR_OPTIONS_MAC_AGE_LIMIT))
+
+        self.assertEqual('0', options.get('mac_binding_removal_limit', '0'))
+
+        cfg.CONF.set_override("mac_binding_age_threshold", 5, group="ovn")
+        cfg.CONF.set_override("mac_binding_removal_limit", 100,
+                              group="ovn_nb_global")
+
+        # Call the maintenance task and check that the value has been
+        # updated in the Logical Switch
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.update_mac_aging_settings)
+
+        lr = self.nb_api.get_lrouter(router['id'])
+        options = self.nb_api.nb_global.options
+
+        self.assertEqual(
+            '5', lr.options.get(ovn_const.LR_OPTIONS_MAC_AGE_LIMIT))
+        self.assertEqual('100', options['mac_binding_removal_limit'])
+
     def test_floating_ip(self):
         ext_net = self._create_network('ext_networktest', external=True)
         ext_subnet = self._create_subnet(
@@ -913,65 +986,69 @@ class TestMaintenance(_TestMaintenanceHelper):
         p1 = self._create_port('testp1', net1['id'])
         p1_ip = p1['fixed_ips'][0]['ip_address']
 
-        with mock.patch('neutron_lib.callbacks.registry.publish') as m_publish:
-            # > Create
-            fip_pf_args = {
-                pf_def.EXTERNAL_PORT: 2222,
-                pf_def.INTERNAL_PORT: 22,
-                pf_def.INTERNAL_PORT_ID: p1['id'],
-                pf_def.PROTOCOL: 'tcp',
-                pf_def.INTERNAL_IP_ADDRESS: p1_ip}
-            pf_obj = self.pf_plugin.create_floatingip_port_forwarding(
-                self.context, fip_id, **fip_attrs(fip_pf_args))
-            call = mock.call('port_forwarding', 'after_create', self.pf_plugin,
-                             payload=mock.ANY)
-            m_publish.assert_has_calls([call])
+        callbacks = registry._get_callback_manager()._callbacks
+        pf_cb = callbacks[pf_consts.PORT_FORWARDING]
+        key = list(pf_cb[events.AFTER_UPDATE][0][1].keys())[0]
+        pf_cb[events.AFTER_CREATE][0][1][key] = mock.MagicMock()
 
-            # Assert load balancer for port forwarding was not created
-            self.assertFalse(self._find_pf_lb(router_id, fip_id))
+        # > Create
+        fip_pf_args = {
+            pf_def.EXTERNAL_PORT: 2222,
+            pf_def.INTERNAL_PORT: 22,
+            pf_def.INTERNAL_PORT_ID: p1['id'],
+            pf_def.PROTOCOL: 'tcp',
+            pf_def.INTERNAL_IP_ADDRESS: p1_ip}
+        pf_obj = self.pf_plugin.create_floatingip_port_forwarding(
+            self.context, fip_id, **fip_attrs(fip_pf_args))
+        call = mock.call('port_forwarding', 'after_create', self.pf_plugin,
+                         payload=mock.ANY)
+        pf_cb[events.AFTER_CREATE][0][1][key].assert_has_calls([call])
 
-            # Call the maintenance thread to fix the problem
-            self.maint.check_for_inconsistencies()
+        # Assert load balancer for port forwarding was not created
+        self.assertFalse(self._find_pf_lb(router_id, fip_id))
 
-            # Assert load balancer for port forwarding was created
-            _verify_lb(self, 'tcp', 2222, 22)
+        # Call the maintenance thread to fix the problem
+        self.maint.check_for_inconsistencies()
 
-            # > Update
-            fip_pf_args = {pf_def.EXTERNAL_PORT: 5353,
-                           pf_def.INTERNAL_PORT: 53,
-                           pf_def.PROTOCOL: 'udp'}
-            m_publish.reset_mock()
-            self.pf_plugin.update_floatingip_port_forwarding(
-                self.context, pf_obj['id'], fip_id, **fip_attrs(fip_pf_args))
-            call = mock.call('port_forwarding', 'after_update', self.pf_plugin,
-                             payload=mock.ANY)
-            m_publish.assert_has_calls([call])
+        # Assert load balancer for port forwarding was created
+        _verify_lb(self, 'tcp', 2222, 22)
 
-            # Assert load balancer for port forwarding is stale
-            _verify_lb(self, 'tcp', 2222, 22)
+        # > Update
+        fip_pf_args = {pf_def.EXTERNAL_PORT: 5353,
+                       pf_def.INTERNAL_PORT: 53,
+                       pf_def.PROTOCOL: 'udp'}
+        pf_cb[events.AFTER_UPDATE][0][1][key] = mock.MagicMock()
+        self.pf_plugin.update_floatingip_port_forwarding(
+            self.context, pf_obj['id'], fip_id, **fip_attrs(fip_pf_args))
+        call = mock.call('port_forwarding', 'after_update', self.pf_plugin,
+                         payload=mock.ANY)
+        pf_cb[events.AFTER_UPDATE][0][1][key].assert_has_calls([call])
 
-            # Call the maintenance thread to fix the problem
-            self.maint.check_for_inconsistencies()
+        # Assert load balancer for port forwarding is stale
+        _verify_lb(self, 'tcp', 2222, 22)
 
-            # Assert load balancer for port forwarding was updated
-            _verify_lb(self, 'udp', 5353, 53)
+        # Call the maintenance thread to fix the problem
+        self.maint.check_for_inconsistencies()
 
-            # > Delete
-            m_publish.reset_mock()
-            self.pf_plugin.delete_floatingip_port_forwarding(
-                self.context, pf_obj['id'], fip_id)
-            call = mock.call('port_forwarding', 'after_delete', self.pf_plugin,
-                             payload=mock.ANY)
-            m_publish.assert_has_calls([call])
+        # Assert load balancer for port forwarding was updated
+        _verify_lb(self, 'udp', 5353, 53)
 
-            # Assert load balancer for port forwarding is stale
-            _verify_lb(self, 'udp', 5353, 53)
+        # > Delete
+        pf_cb[events.AFTER_DELETE][0][1][key] = mock.MagicMock()
+        self.pf_plugin.delete_floatingip_port_forwarding(
+            self.context, pf_obj['id'], fip_id)
+        call = mock.call('port_forwarding', 'after_delete', self.pf_plugin,
+                         payload=mock.ANY)
+        pf_cb[events.AFTER_DELETE][0][1][key].assert_has_calls([call])
 
-            # Call the maintenance thread to fix the problem
-            self.maint.check_for_inconsistencies()
+        # Assert load balancer for port forwarding is stale
+        _verify_lb(self, 'udp', 5353, 53)
 
-            # Assert load balancer for port forwarding is gone
-            self.assertFalse(self._find_pf_lb(router_id, fip_id))
+        # Call the maintenance thread to fix the problem
+        self.maint.check_for_inconsistencies()
+
+        # Assert load balancer for port forwarding is gone
+        self.assertFalse(self._find_pf_lb(router_id, fip_id))
 
     def test_check_for_ha_chassis_group(self):
         net1 = self._create_network('network1test', external=False)
@@ -1108,6 +1185,39 @@ class TestMaintenance(_TestMaintenanceHelper):
     def test_check_provider_distributed_ports_non_dvr_geneve_net(self):
         self._test_check_provider_distributed_ports(False, 'geneve')
 
+    def test_configure_nb_global(self):
+        def options_intersect(options1, options2):
+            return bool(set(
+                new_nb_global_options.keys()).intersection(nb_options.keys()))
+
+        new_nb_global_options = {
+            'foo': 'bar',
+            'baz': 'qux',
+        }
+
+        cfg_nb_global_options = [
+            ovn_config.cfg.StrOpt(key) for key in new_nb_global_options
+        ]
+
+        def get_opt(key):
+            return new_nb_global_options[key]
+
+        nb_options = self.nb_api.db_get('NB_Global', '.', 'options').execute(
+            check_error=True, log_errors=True)
+        self.assertFalse(options_intersect(new_nb_global_options, nb_options))
+
+        with mock.patch.object(
+                ovn_config, 'nb_global_opts', cfg_nb_global_options), \
+                mock.patch.object(
+                    ovn_config.cfg.CONF.ovn_nb_global, 'get',
+                    side_effect=get_opt):
+            self.assertRaises(periodics.NeverAgain,
+                              self.maint.configure_nb_global)
+
+        nb_options = self.nb_api.db_get('NB_Global', '.', 'options').execute(
+            check_error=True, log_errors=True)
+        self.assertTrue(options_intersect(new_nb_global_options, nb_options))
+
     def test_floating_ip_with_gateway_port(self):
         ext_net = self._create_network('ext_networktest', external=True)
         ext_subnet = self._create_subnet(
@@ -1167,6 +1277,62 @@ class TestMaintenance(_TestMaintenanceHelper):
         else:
             self.assertNotIn('gateway_port', fip_rule)
 
+    def test_add_provider_resource_association_to_routers(self):
+        provider_name = 'ovn'
+        router1 = self._create_router(uuidutils.generate_uuid())
+        router2 = self._create_router(uuidutils.generate_uuid())
+
+        # NOTE(ralonsoh): now each Neutron router will create a
+        # ``ProviderResourceAssociation`` register. In order to be able to test
+        # the maintenance method, the router2 ``ProviderResourceAssociation``
+        # register is deleted.
+        servicetype_obj.ProviderResourceAssociation.delete_objects(
+            self.context, resource_id=router2['id'])
+        pra_list = servicetype_obj.ProviderResourceAssociation.get_objects(
+            self.context, provider_name=provider_name)
+        pra_res_ids = set(pra.resource_id for pra in pra_list)
+        self.assertIn(router1['id'], pra_res_ids)
+        self.assertNotIn(router2['id'], pra_res_ids)
+
+        self.assertRaises(
+            periodics.NeverAgain,
+            self.maint.add_provider_resource_association_to_routers)
+        pra_list = servicetype_obj.ProviderResourceAssociation.get_objects(
+            self.context, provider_name=provider_name)
+        pra_res_ids = set(pra.resource_id for pra in pra_list)
+        self.assertIn(router1['id'], pra_res_ids)
+        self.assertIn(router2['id'], pra_res_ids)
+
+    def test_remove_invalid_gateway_chassis_from_unbound_lrp(self):
+        net1 = self._create_network(uuidutils.generate_uuid(), external=True)
+        subnet1 = self._create_subnet(uuidutils.generate_uuid(), net1['id'])
+        external_gateway_info = {
+            'enable_snat': True, 'network_id': net1['id'],
+            'external_fixed_ips': [{'ip_address': '10.0.0.2',
+                                    'subnet_id': subnet1['id']}]}
+        router = self._create_router(
+            uuidutils.generate_uuid(),
+            external_gateway_info=external_gateway_info)
+
+        # Manually add the LRP.gateway_chassis with name
+        # 'neutron-ovn-invalid-chassis'
+        lr = self.nb_api.lookup('Logical_Router', utils.ovn_name(router['id']))
+        lrp = lr.ports[0]
+        self.nb_api.lrp_set_gateway_chassis(
+            lrp.uuid, 'neutron-ovn-invalid-chassis').execute(check_error=True)
+        gc = self.nb_api.db_find_rows(
+            'Gateway_Chassis',
+            ('chassis_name', '=', 'neutron-ovn-invalid-chassis')).execute(
+            check_error=True)[0]
+
+        self.assertRaises(
+            periodics.NeverAgain,
+            self.maint.remove_invalid_gateway_chassis_from_unbound_lrp)
+        self.assertIsNone(self.nb_api.lookup('Gateway_Chassis', gc.uuid,
+                                             default=None))
+        lr = self.nb_api.lookup('Logical_Router', utils.ovn_name(router['id']))
+        self.assertEqual([], lr.ports[0].gateway_chassis)
+
     def _get_nb_global_external_ids(self):
         return self.nb_api.db_get(
             'NB_Global', '.', 'external_ids').execute(check_error=True)
