@@ -19,11 +19,13 @@ from neutron_lib.api.definitions import dns as dns_apidef
 from neutron_lib.api.definitions import fip_pf_description as ext_pf_def
 from neutron_lib.api.definitions import fip_pf_port_range as ranges_pf_def
 from neutron_lib.api.definitions import floating_ip_port_forwarding as pf_def
-from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import port_security as ps
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.db import api as db_api
+from neutron_lib.services.logapi import constants as log_const
 from neutron_lib.services.qos import constants as qos_const
+from oslo_config import cfg
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import constants as ovsdbapp_const
@@ -56,6 +58,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
         super(TestOvnNbSync, self).setUp(maintenance_worker=True)
         self.assertEqual(mysql_dialect.name, self.db.engine.dialect.name)
         ovn_config.cfg.CONF.set_override('dns_domain', 'ovn.test')
+        cfg.CONF.set_override('quota_security_group_rule', -1, group='QUOTAS')
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
         self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
         sg_mgr = test_securitygroup.SecurityGroupTestExtensionManager()
@@ -104,7 +107,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             nb_idl=self.nb_api)
 
     def get_additional_service_plugins(self):
-        return {'qos': 'qos', 'segments': 'segments'}
+        return {'qos': 'qos', 'segments': 'segments', 'log': 'log'}
 
     def _api_for_resource(self, resource):
         if resource in ['security-groups']:
@@ -780,11 +783,11 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             for lrouter_name in self.create_lrouters:
                 external_ids = {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
                                 lrouter_name}
-                txn.add(self.nb_api.create_lrouter(lrouter_name, True,
-                                                   external_ids=external_ids))
+                txn.add(self.nb_api.lr_add(router=lrouter_name, may_exist=True,
+                                           external_ids=external_ids))
 
             for lrouter_name in self.delete_lrouters:
-                txn.add(self.nb_api.delete_lrouter(lrouter_name, True))
+                txn.add(self.nb_api.lr_del(lrouter_name, if_exists=True))
 
             for lrport, lrouter_name in self.create_lrouter_ports:
                 txn.add(self.nb_api.add_lrouter_port(lrport, lrouter_name))
@@ -1073,7 +1076,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
 
     @staticmethod
     def _build_acl_for_pgs(priority, direction, log, name, action,
-                           severity, match, port_group, **kwargs):
+                           severity, match, meter, port_group, **kwargs):
         return {
             'priority': priority,
             'direction': direction,
@@ -1082,6 +1085,7 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             'action': action,
             'severity': severity,
             'match': match,
+            'meter': meter,
             'external_ids': kwargs}
 
     def _validate_dhcp_opts(self, should_match=True):
@@ -1154,6 +1158,9 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
                 ovn_const.OVN_DROP_PORT_GROUP_NAME):
             db_acls.append(TestOvnNbSync._build_acl_for_pgs(**acl))
 
+        self.ovn_log_driver.add_logging_options_to_acls(db_acls,
+                                                        self.ctx)
+
         # Get the list of ACLs stored in the OVN plugin IDL.
         plugin_acls = []
         for row in _plugin_nb_ovn._tables['Logical_Switch'].rows.values():
@@ -1174,7 +1181,24 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
             for acl in getattr(row, 'acls', []):
                 monitor_acls.append(self._build_acl_to_compare(acl))
 
+        self.ovn_log_driver.add_logging_options_to_acls(monitor_acls,
+                                                        self.ctx)
+        self.ovn_log_driver.add_logging_options_to_acls(plugin_acls,
+                                                        self.ctx)
+
+        # Values taken out from list for comparison, since ACLs from OVN DB
+        # have certain values on a list of just one object
         if should_match:
+            for acl in plugin_acls:
+                if isinstance(acl['severity'], list) and acl['severity']:
+                    acl['severity'] = acl['severity'][0]
+                    acl['name'] = acl['name'][0]
+                    acl['meter'] = acl['meter'][0]
+            for acl in monitor_acls:
+                if isinstance(acl['severity'], list) and acl['severity']:
+                    acl['severity'] = acl['severity'][0]
+                    acl['name'] = acl['name'][0]
+                    acl['meter'] = acl['meter'][0]
             self.assertCountEqual(db_acls, plugin_acls)
             self.assertCountEqual(db_acls, monitor_acls)
         else:
@@ -1196,10 +1220,10 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
                                           db_route['nexthop']
                                           for db_route in db_router['routes']]
             db_nats[db_router['id']] = []
-            if db_router.get(l3.EXTERNAL_GW_INFO):
-                gateways = self.l3_plugin._ovn_client._get_gw_info(
-                    self.context, db_router)
-                for gw_info in gateways:
+            for gw_port in self.l3_plugin._ovn_client._get_router_gw_ports(
+                    self.context, db_router['id']):
+                for gw_info in self.l3_plugin._ovn_client._get_gw_info(
+                        self.context, gw_port):
                     # Add gateway default route and snats
                     if gw_info.gateway_ip:
                         db_routes[db_router['id']].append(gw_info.ip_prefix +
@@ -1755,61 +1779,82 @@ class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
         nb_synchronizer.sync_fip_qos_policies(ctx)
         self._validate_qos_records()
 
-    def test_fip_nat_revert_to_stateful(self):
-        res = self._create_network(self.fmt, 'n1_ext', True, as_admin=True,
-                                   arg_list=('router:external', ),
-                                   **{'router:external': True})
-        net_ext = self.deserialize(self.fmt, res)['network']
-        res = self._create_subnet(self.fmt, net_ext['id'], '10.0.0.0/24')
-        subnet_ext = self.deserialize(self.fmt, res)['subnet']
+    def _create_security_group_rule(self, sg_id, direction, tcp_port):
+        data = {'security_group_rule': {'security_group_id': sg_id,
+                                        'direction': direction,
+                                        'protocol': constants.PROTO_NAME_TCP,
+                                        'ethertype': constants.IPv4,
+                                        'port_range_min': tcp_port,
+                                        'port_range_max': tcp_port}}
+        req = self.new_create_request('security-group-rules', data, self.fmt)
+        res = req.get_response(self.api)
+        sgr = self.deserialize(self.fmt, res)
+        self.assertIn('security_group_rule', sgr)
+        return sgr['security_group_rule']['id']
 
-        res = self._create_network(self.fmt, 'n1_int', True)
-        net_int = self.deserialize(self.fmt, res)['network']
-        self._create_subnet(self.fmt, net_int['id'], '10.10.0.0/24')
+    def _test_sync_acls_helper(self, test_log=False,
+                               log_event=log_const.ALL_EVENT):
+        data = {'security_group': {'name': 'sg1'}}
+        sg_req = self.new_create_request('security-groups', data)
+        res = sg_req.get_response(self.api)
+        sg = self.deserialize(self.fmt, res)['security_group']
 
-        port = self._make_port(self.fmt, net_int['id'],
-                               name='test-port')['port']
+        sgr_ids = []
 
-        data = {'name': 'r1', 'admin_state_up': True,
-                'tenant_id': self._tenant_id,
-                'external_gateway_info': {
-                    'enable_snat': True,
-                    'network_id': net_ext['id'],
-                    'external_fixed_ips': [{'ip_address': '10.0.0.5',
-                                            'subnet_id': subnet_ext['id']}]}
-                }
-        router = self.l3_plugin.create_router(self.context, {'router': data})
-        self.l3_plugin.add_router_interface(
-            self.context, router['id'], {'port_id': port['id']})
+        # If we are going to test ACLs with log enabled, set up a log object
+        if test_log:
+            log_data = {'log': {'project_id': self.ctx.project_id,
+            'resource_type': 'security_group',
+            'description': 'test net log',
+            'name': 'logme',
+            'enabled': True,
+            'event': log_event}}
+            log_obj = self.log_plugin.create_log(self.ctx, log_data)
 
-        body = {'tenant_id': self._tenant_id,
-                'floating_network_id': net_ext['id'],
-                'port_id': port['id']}
-        self.l3_plugin.create_floatingip(self.context, {'floatingip': body})
-
-        self.assertEqual(0, len(self.nb_api.get_all_stateless_fip_nats()))
-
-        def get_all_stateful_fip_nats():
-            cmd = self.nb_api.db_find('NAT',
-                ('external_ids', '!=', {ovn_const.OVN_FIP_EXT_ID_KEY: ''}),
-                ('options', '=', {}),
-                ('type', '=', 'dnat_and_snat'))
-            return cmd.execute(check_error=True)
+        for tcp_port in range(8050, 8055):
+            sgr_ids.append(self._create_security_group_rule(
+                sg['id'], 'ingress', tcp_port))
+        for tcp_port in range(10000, 10005):
+            sgr_ids.append(self._create_security_group_rule(
+                sg['id'], 'egress', tcp_port))
+        self._validate_acls()
 
+        # Delete ACLs from the OVN DB.
         with self.nb_api.transaction(check_error=True) as txn:
-            for nat in get_all_stateful_fip_nats():
-                txn.add(self.nb_api.db_set(
-                    'NAT', nat['_uuid'],
-                    ('options', {'stateless': 'true'})))
-
-        self.assertEqual(1, len(self.nb_api.get_all_stateless_fip_nats()))
+            pg_name = utils.ovn_port_group_name(sg['id'])
+            txn.add(self.nb_api.pg_acl_del(pg_name, direction='to-lport'))
+        self._validate_acls(should_match=False)
 
         nb_synchronizer = ovn_db_sync.OvnNbSynchronizer(
             self.plugin, self.mech_driver.nb_ovn, self.mech_driver.sb_ovn,
             'repair', self.mech_driver)
-        nb_synchronizer.migrate_to_stateful_fips(self.context)
+        ctx = context.get_admin_context()
+        nb_synchronizer.sync_acls(ctx)
+        self._validate_acls()
+        # Delete Security Group Rules from Neutron DB.
+        for i in range(5):
+            with db_api.CONTEXT_WRITER.using(context):
+                sgr = self.plugin._get_security_group_rule(
+                    self.ctx, sgr_ids[i])
+                sgr.delete()
+        self._validate_acls(should_match=False)
+        nb_synchronizer.sync_acls(ctx)
+        self._validate_acls()
 
-        self.assertEqual(0, len(self.nb_api.get_all_stateless_fip_nats()))
+        # Remove log object to avoid overlapping
+        if test_log:
+            log_obj = self.log_plugin.delete_log(self.ctx, log_obj['id'])
+
+    def test_sync_acls(self):
+        self._test_sync_acls_helper()
+
+    def test_sync_acls_with_logging(self):
+        self._test_sync_acls_helper(test_log=True,
+                                    log_event=log_const.ACCEPT_EVENT)
+        self._test_sync_acls_helper(test_log=True,
+                                    log_event=log_const.ALL_EVENT)
+        self._test_sync_acls_helper(test_log=True,
+                                    log_event=log_const.DROP_EVENT)
 
 
 class TestOvnSbSync(base.TestOVNFunctionalBase):
