@@ -302,6 +302,37 @@ class ProjectReaderSecurityGroupTests(ProjectMemberSecurityGroupTests):
             self.context, 'delete_security_groups_tags', self.alt_target)
 
 
+class ServiceRoleSecurityGroupTests(SecurityGroupAPITestCase):
+
+    def setUp(self):
+        super(ServiceRoleSecurityGroupTests, self).setUp()
+        self.context = self.service_ctx
+
+    def test_create_security_group(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_group', self.target)
+
+    def test_get_security_group(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_security_group', self.target)
+
+    def test_update_security_group(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_security_group', self.target)
+
+    def test_delete_security_group(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_security_group', self.target)
+
+
 class SecurityGroupRuleAPITestCase(base.PolicyBaseTestCase):
 
     def setUp(self):
@@ -341,6 +372,24 @@ class SecurityGroupRuleAPITestCase(base.PolicyBaseTestCase):
             'neutron_lib.plugins.directory.get_plugin',
             return_value=self.plugin_mock).start()
 
+    def override_create_security_group_rule(self):
+        self._override_security_group_rule('create_security_group_rule')
+
+    def override_delete_security_group_rule(self):
+        self._override_security_group_rule('delete_security_group_rule')
+
+    def _override_security_group_rule(self, rule_name):
+        # Admin or (member and not default SG) --> only admin can perform the
+        # ``rule_name`` action in the default SG.
+        rule = {rule_name:
+                'role:admin or (role:member and project_id:%(project_id)s '
+                'and not rule:rule_default_sg)'}
+        policy_file = base.write_policies(rule)
+        self.target['belongs_to_default_sg'] = 'True'
+        base.reload_policies(policy_file)
+        self.plugin_mock.get_default_security_group.return_value = (
+            self.sg['id'])
+
 
 class SystemAdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
 
@@ -407,6 +456,15 @@ class AdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
             policy.enforce(self.context,
                            'create_security_group_rule', self.alt_target))
 
+    def test_create_security_group_rule_default_sg(self):
+        self.override_create_security_group_rule()
+        self.assertTrue(
+            policy.enforce(self.context,
+                           'create_security_group_rule', self.target))
+        self.assertTrue(
+            policy.enforce(self.context,
+                           'create_security_group_rule', self.alt_target))
+
     def test_get_security_group_rule(self):
         self.assertTrue(
             policy.enforce(self.context,
@@ -423,6 +481,15 @@ class AdminSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
             policy.enforce(self.context,
                            'delete_security_group_rule', self.alt_target))
 
+    def test_delete_security_group_rule_default_sg(self):
+        self.override_delete_security_group_rule()
+        self.assertTrue(
+            policy.enforce(self.context,
+                           'delete_security_group_rule', self.target))
+        self.assertTrue(
+            policy.enforce(self.context,
+                           'delete_security_group_rule', self.alt_target))
+
 
 class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
 
@@ -450,6 +517,17 @@ class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
             policy.enforce,
             self.context, 'create_security_group_rule', target)
 
+    def test_create_security_group_rule_default_sg(self):
+        self.override_create_security_group_rule()
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_group_rule', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_group_rule', self.alt_target)
+
     def test_get_security_group_rule(self):
         self.assertTrue(
             policy.enforce(self.context,
@@ -480,6 +558,17 @@ class ProjectMemberSecurityGroupRuleTests(AdminSecurityGroupRuleTests):
             policy.enforce,
             self.context, 'delete_security_group_rule', target)
 
+    def test_delete_security_group_rule_default_sg(self):
+        self.override_delete_security_group_rule()
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_security_group_rule', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_security_group_rule', self.alt_target)
+
 
 class ProjectReaderSecurityGroupRuleTests(ProjectMemberSecurityGroupRuleTests):
 
@@ -516,7 +605,6 @@ class ProjectReaderSecurityGroupRuleTests(ProjectMemberSecurityGroupRuleTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce,
             self.context, 'delete_security_group_rule', self.alt_target)
-
         # Test for the SG_OWNER different then current user case:
         target = copy.copy(self.target)
         target['security_group_id'] = self.alt_sg['id']
@@ -527,3 +615,28 @@ class ProjectReaderSecurityGroupRuleTests(ProjectMemberSecurityGroupRuleTests):
             base_policy.PolicyNotAuthorized,
             policy.enforce,
             self.context, 'delete_security_group_rule', target)
+
+
+class ServiceRoleSecurityGroupRuleTests(SecurityGroupRuleAPITestCase):
+
+    def setUp(self):
+        super(ServiceRoleSecurityGroupRuleTests, self).setUp()
+        self.context = self.service_ctx
+
+    def test_create_security_group_rule(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'create_security_group_rule', self.target)
+
+    def test_get_security_group_rule(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_security_group_rule', self.target)
+
+    def test_delete_security_group_rule(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_security_group_rule', self.target)
