@@ -15,12 +15,16 @@
 
 from unittest import mock
 
+from neutron_lib import context as ncontext
+from oslo_config import cfg
+
 from neutron.common.ovn import constants
 from neutron.conf.plugins.ml2 import config as ml2_conf
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.tests import base
+from neutron.tests.unit import fake_resources as fakes
 from neutron.tests.unit.services.logapi.drivers.ovn \
     import test_driver as test_log_driver
 from neutron_lib.api.definitions import l3
@@ -31,6 +35,53 @@ from neutron_lib.services.trunk import constants as trunk_const
 from tenacity import wait_none
 
 
+class Test_has_separate_snat_per_subnet(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        ovn_conf.register_opts()
+
+    def test_snat_on_nested_off(self):
+        fake_router = {
+            'id': 'fake-id',
+            l3.EXTERNAL_GW_INFO: {
+                'enable_snat': True,
+            },
+        }
+        # ovn_router_indirect_snat default is False
+        self.assertTrue(ovn_client._has_separate_snat_per_subnet(fake_router))
+
+    def test_snat_off_nested_off(self):
+        fake_router = {
+            'id': 'fake-id',
+            l3.EXTERNAL_GW_INFO: {
+                'enable_snat': False,
+            },
+        }
+        # ovn_router_indirect_snat default is False
+        self.assertFalse(ovn_client._has_separate_snat_per_subnet(fake_router))
+
+    def test_snat_on_nested_on(self):
+        fake_router = {
+            'id': 'fake-id',
+            l3.EXTERNAL_GW_INFO: {
+                'enable_snat': True,
+            },
+        }
+        cfg.CONF.set_override('ovn_router_indirect_snat', True, 'ovn')
+        self.assertFalse(ovn_client._has_separate_snat_per_subnet(fake_router))
+
+    def test_snat_off_nested_on(self):
+        fake_router = {
+            'id': 'fake-id',
+            l3.EXTERNAL_GW_INFO: {
+                'enable_snat': False,
+            },
+        }
+        cfg.CONF.set_override('ovn_router_indirect_snat', True, 'ovn')
+        self.assertFalse(ovn_client._has_separate_snat_per_subnet(fake_router))
+
+
 class TestOVNClientBase(base.BaseTestCase):
 
     def setUp(self):
@@ -56,60 +107,138 @@ class TestOVNClient(TestOVNClientBase):
         plugin = mock.MagicMock()
         self.get_plugin.return_value = plugin
         subnet = {
-            'subnet_id': 'fake-subnet-id',
+            'id': 'fake-subnet-id',
             'gateway_ip': '10.42.0.1',
             'ip_version': const.IP_VERSION_4,
         }
         plugin.get_subnet.return_value = subnet
+        plugin.get_subnets_by_network.return_value = [subnet]
         router = {
             'id': 'fake-router-id',
-            l3.EXTERNAL_GW_INFO: {
-                'external_fixed_ips': [{
-                        'subnet_id': subnet.get('subnet_id'),
-                        'ip_address': '10.42.0.42'}],
-            },
             'gw_port_id': 'fake-port-id',
         }
-        networks = mock.MagicMock()
         txn = mock.MagicMock()
+        self.ovn_client._get_router_gw_ports = mock.MagicMock()
+        gw_port = fakes.FakePort().create_one_port(
+            attrs={
+                'id': router['gw_port_id'],
+                'fixed_ips': [{
+                    'subnet_id': subnet.get('id'),
+                    'ip_address': '10.42.0.42'}]
+            })
+        self.ovn_client._get_router_gw_ports.return_value = [gw_port]
         self.assertEqual(
-            self.get_plugin().get_port(),
-            self.ovn_client._add_router_ext_gw(mock.Mock(), router, networks,
-                                               txn))
+            [self.get_plugin().get_port()],
+            self.ovn_client._add_router_ext_gw(mock.Mock(), router, txn))
         self.nb_idl.add_static_route.assert_called_once_with(
             'neutron-' + router['id'],
             ip_prefix='0.0.0.0/0',
             nexthop='10.42.0.1',
+            maintain_bfd=False,
             external_ids={
                 'neutron:is_ext_gw': 'true',
-                'neutron:subnet_id': subnet['subnet_id']})
+                'neutron:subnet_id': subnet['id']})
+
+    def test__add_router_ext_gw_default_route_ecmp(self):
+        plugin = mock.MagicMock()
+        self.get_plugin.return_value = plugin
+        subnet1 = {
+            'id': 'fake-subnet-id-1',
+            'gateway_ip': '10.42.0.1',
+            'ip_version': const.IP_VERSION_4,
+        }
+        subnet2 = {
+            'id': 'fake-subnet-id-2',
+            'gateway_ip': '10.42.42.1',
+            'ip_version': const.IP_VERSION_4,
+        }
+        plugin.get_subnet.side_effect = [subnet1, subnet2, subnet1, subnet2]
+        plugin.get_subnets_by_network.return_value = [subnet1, subnet2]
+        router = {
+            'id': 'fake-router-id',
+            'gw_port_id': 'fake-port-id',
+            'enable_default_route_ecmp': True,
+        }
+        txn = mock.MagicMock()
+        self.ovn_client._get_router_gw_ports = mock.MagicMock()
+        gw_port1 = fakes.FakePort().create_one_port(
+            attrs={
+                'id': router['gw_port_id'],
+                'fixed_ips': [{
+                    'subnet_id': subnet1.get('id'),
+                    'ip_address': '10.42.0.42'}]
+            })
+        gw_port2 = fakes.FakePort().create_one_port(
+            attrs={
+                'id': 'gw-port-id-2',
+                'fixed_ips': [{
+                    'subnet_id': subnet2.get('id'),
+                    'ip_address': '10.42.42.42'}]
+            })
+        self.ovn_client._get_router_gw_ports.return_value = [
+            gw_port1, gw_port2]
+        self.assertEqual(
+            [self.get_plugin().get_port(), self.get_plugin().get_port()],
+            self.ovn_client._add_router_ext_gw(mock.Mock(), router, txn))
+        self.nb_idl.add_static_route.assert_has_calls([
+            mock.call('neutron-' + router['id'],
+                      ip_prefix='0.0.0.0/0',
+                      nexthop=subnet1['gateway_ip'],
+                      maintain_bfd=False,
+                      external_ids={
+                         'neutron:is_ext_gw': 'true',
+                         'neutron:subnet_id': subnet1['id']},
+                      ),
+            mock.call('neutron-' + router['id'],
+                      ip_prefix='0.0.0.0/0',
+                      nexthop=subnet2['gateway_ip'],
+                      maintain_bfd=False,
+                      external_ids={
+                         'neutron:is_ext_gw': 'true',
+                         'neutron:subnet_id': subnet2['id']},
+                      ),
+        ])
 
     def test__add_router_ext_gw_no_default_route(self):
         plugin = mock.MagicMock()
         self.get_plugin.return_value = plugin
         subnet = {
-            'subnet_id': 'fake-subnet-id',
+            'id': 'fake-subnet-id',
             'gateway_ip': None,
             'ip_version': const.IP_VERSION_4
         }
         plugin.get_subnet.return_value = subnet
+        plugin.get_subnets_by_network.return_value = [subnet]
         router = {
             'id': 'fake-router-id',
             l3.EXTERNAL_GW_INFO: {
                 'external_fixed_ips': [{
-                        'subnet_id': subnet.get('subnet_id'),
+                        'subnet_id': subnet.get('id'),
                         'ip_address': '10.42.0.42'}],
             },
             'gw_port_id': 'fake-port-id',
         }
-        networks = mock.MagicMock()
         txn = mock.MagicMock()
+        self.ovn_client._get_router_gw_ports = mock.MagicMock()
+        gw_port = fakes.FakePort().create_one_port(
+            attrs={
+                'id': router['gw_port_id'],
+                'fixed_ips': [{
+                    'subnet_id': subnet.get('id'),
+                    'ip_address': '10.42.0.42'}]
+            })
+        self.ovn_client._get_router_gw_ports.return_value = [gw_port]
         self.assertEqual(
-            self.get_plugin().get_port(),
-            self.ovn_client._add_router_ext_gw(mock.Mock(), router, networks,
-                                               txn))
+            [self.get_plugin().get_port()],
+            self.ovn_client._add_router_ext_gw(mock.Mock(), router, txn))
         self.nb_idl.add_static_route.assert_not_called()
 
+    def test_checkout_ip_list(self):
+        addresses = ["192.168.2.2/32", "2001:db8::/32"]
+        add_map = self.ovn_client._checkout_ip_list(addresses)
+        self.assertEqual(["192.168.2.2/32"], add_map[const.IP_VERSION_4])
+        self.assertEqual(["2001:db8::/32"], add_map[const.IP_VERSION_6])
+
     def test_update_lsp_host_info_up(self):
         context = mock.MagicMock()
         host_id = 'fake-binding-host-id'
@@ -224,6 +353,27 @@ class TestOVNClient(TestOVNClientBase):
                           mock.call(context, port_id)]
         mock_get_port.assert_has_calls(expected_calls)
 
+    def test__get_snat_cidrs_for_external_router_nested_snat_off(self):
+        ctx = ncontext.Context()
+        per_subnet_cidrs = ['10.0.0.0/24', '20.0.0.0/24']
+        with mock.patch.object(
+                self.ovn_client, '_get_v4_network_of_all_router_ports',
+                return_value=per_subnet_cidrs):
+            cidrs = self.ovn_client._get_snat_cidrs_for_external_router(
+                ctx, 'fake-id')
+        self.assertEqual(per_subnet_cidrs, cidrs)
+
+    def test__get_snat_cidrs_for_external_router_nested_snat_on(self):
+        ctx = ncontext.Context()
+        cfg.CONF.set_override('ovn_router_indirect_snat', True, 'ovn')
+        per_subnet_cidrs = ['10.0.0.0/24', '20.0.0.0/24']
+        with mock.patch.object(
+                self.ovn_client, '_get_v4_network_of_all_router_ports',
+                return_value=per_subnet_cidrs):
+            cidrs = self.ovn_client._get_snat_cidrs_for_external_router(
+                ctx, 'fake-id')
+        self.assertEqual([constants.OVN_DEFAULT_SNAT_CIDR], cidrs)
+
 
 class TestOVNClientFairMeter(TestOVNClientBase,
                              test_log_driver.TestOVNDriverBase):
