@@ -16,6 +16,8 @@ import collections
 from unittest import mock
 
 from neutron_lib import constants as const
+from neutron_lib.services.logapi import constants as log_const
+from oslo_utils import uuidutils
 
 from neutron.common.ovn import acl
 from neutron.common.ovn import constants as ovn_const
@@ -227,7 +229,10 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
               'lswitch': 'lswitch1', 'lport': 'lport1'}],
             'lport2':
             [{'id': 'acl2', 'priority': 00, 'policy': 'drop',
-             'lswitch': 'lswitch2', 'lport': 'lport2'}],
+             'lswitch': 'lswitch2', 'lport': 'lport2'},
+             {'id': 'aclr3', 'priority': 00, 'log': True,
+              'policy': 'drop', 'lswitch': 'lswitch2',
+              'meter': 'acl_log_meter', 'label': 1, 'lport': 'lport2'}],
             # ACLs need to be kept as-is by the sync tool
             'p2n2':
             [{'lport': 'p2n2', 'direction': 'to-lport',
@@ -263,7 +268,9 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                              'external_fixed_ips': [
                                  {'subnet_id': 'ext-subnet',
                                   'ip_address': '100.0.0.2'}]}},
-                        {'id': 'r4', 'routes': []}]
+                        {'id': 'r4', 'routes': []},
+                        {'id': 'r5', 'routes': [],
+                         'flavor_id': 'user-defined'}]
 
         self.get_sync_router_ports = [
             {'fixed_ips': [{'subnet_id': 'subnet1',
@@ -364,25 +371,25 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                     'external_ids': {'subnet_id': 'n1-s1'}}
         return {'cidr': '', 'options': '', 'external_ids': {}}
 
-    def _fake_get_gw_info(self, ctx, router):
+    def _fake_get_gw_info(self, ctx, port):
         return {
-            'r1': [ovn_client.GW_INFO(router_ip='90.0.0.2',
-                                      gateway_ip='90.0.0.1',
-                                      network_id='', subnet_id='ext-subnet',
-                                      ip_version=4,
-                                      ip_prefix=const.IPv4_ANY)],
-            'r2': [ovn_client.GW_INFO(router_ip='100.0.0.2',
-                                      gateway_ip='100.0.0.1',
-                                      network_id='', subnet_id='ext-subnet',
-                                      ip_version=4,
-                                      ip_prefix=const.IPv4_ANY)]
-        }.get(router['id'], [])
+            'p1r1': [ovn_client.GW_INFO(router_ip='90.0.0.2',
+                                        gateway_ip='90.0.0.1',
+                                        network_id='', subnet_id='ext-subnet',
+                                        ip_version=4,
+                                        ip_prefix=const.IPv4_ANY)],
+            'p1r2': [ovn_client.GW_INFO(router_ip='100.0.0.2',
+                                        gateway_ip='100.0.0.1',
+                                        network_id='', subnet_id='ext-subnet',
+                                        ip_version=4,
+                                        ip_prefix=const.IPv4_ANY)]
+        }.get(port['id'], [])
 
-    def _fake_get_v4_network_of_all_router_ports(self, ctx, router_id):
+    def _fake_get_snat_cidrs_for_external_router(self, ctx, router_id):
         return {'r1': ['172.16.0.0/24', '172.16.2.0/24'],
                 'r2': ['192.168.2.0/24']}.get(router_id, [])
 
-    def _test_mocks_helper(self, ovn_nb_synchronizer):
+    def _test_mocks_helper(self, ovn_nb_synchronizer, test_logging=False):
         core_plugin = ovn_nb_synchronizer.core_plugin
         ovn_api = ovn_nb_synchronizer.ovn_api
         ovn_driver = ovn_nb_synchronizer.ovn_driver
@@ -413,12 +420,25 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         # 4 acls are returned as current ovn acls,
         # two of which will match with neutron.
         # So, in this example 17 will be added, 2 removed
+
         core_plugin.get_ports = mock.Mock()
         core_plugin.get_ports.return_value = self.ports
         mock.patch.object(acl, '_get_subnet_from_cache',
                           return_value=self.subnet).start()
         mock.patch.object(acl, 'acl_remote_group_id',
                           side_effect=self.matches).start()
+        if test_logging:
+            log_objs = [self._fake_log_obj(
+                event=log_const.DROP_EVENT, resource_id=None, id='1111')]
+            mock.patch.object(ovn_nb_synchronizer.ovn_log_driver, '_get_logs',
+                              return_value=log_objs).start()
+            mock.patch.object(ovn_nb_synchronizer.ovn_log_driver,
+                              '_pgs_from_log_obj', return_value=[
+                                  {'name': 'neutron_pg_drop',
+                                   'external_ids': {},
+                                   'acls': [uuidutils.generate_uuid()]}]
+                              ).start()
+
         core_plugin.get_security_group = mock.MagicMock(
             side_effect=self.security_groups)
         ovn_nb_synchronizer.get_acls = mock.Mock()
@@ -448,15 +468,14 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         l3_plugin._get_sync_interfaces = mock.Mock()
         l3_plugin._get_sync_interfaces.return_value = (
             self.get_sync_router_ports)
-        ovn_nb_synchronizer._ovn_client = mock.Mock()
-        ovn_nb_synchronizer._ovn_client.\
-            _get_nets_and_ipv6_ra_confs_for_router_port.return_value = (
+        ovn_client = mock.Mock()
+        ovn_nb_synchronizer._ovn_client = ovn_client
+        ovn_client._get_nets_and_ipv6_ra_confs_for_router_port.return_value = (
                 self.lrport_networks, {'fixed_ips': {}})
-        ovn_nb_synchronizer._ovn_client._get_v4_network_of_all_router_ports. \
-            side_effect = self._fake_get_v4_network_of_all_router_ports
-        ovn_nb_synchronizer._ovn_client._get_gw_info = mock.Mock()
-        ovn_nb_synchronizer._ovn_client._get_gw_info.side_effect = (
-            self._fake_get_gw_info)
+        ovn_client._get_snat_cidrs_for_external_router.side_effect = (
+            self._fake_get_snat_cidrs_for_external_router)
+        ovn_client._get_gw_info = mock.Mock()
+        ovn_client._get_gw_info.side_effect = self._fake_get_gw_info
         # end of router-sync block
         l3_plugin.get_floatingips = mock.Mock()
         l3_plugin.get_floatingips.return_value = self.floating_ips
@@ -465,8 +484,6 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         ovn_api.get_all_logical_switches_with_ports.return_value = (
             self.lswitches_with_ports)
 
-        ovn_api.get_all_stateless_fip_nats = mock.Mock()
-        ovn_api.get_all_stateless_fip_nats.return_value = []
         ovn_api.get_all_logical_routers_with_rports = mock.Mock()
         ovn_api.get_all_logical_routers_with_rports.return_value = (
             self.lrouters_with_rports)
@@ -545,6 +562,11 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         ovn_api.delete_dhcp_options = mock.Mock()
         ovn_nb_synchronizer._ovn_client.get_port_dns_records = mock.Mock()
         ovn_nb_synchronizer._ovn_client.get_port_dns_records.return_value = {}
+        ovn_nb_synchronizer._ovn_client._get_router_gw_ports.side_effect = (
+            [self.get_sync_router_ports[0]],
+            [self.get_sync_router_ports[1]],
+            [self.get_sync_router_ports[2]],
+        )
 
     def _test_ovn_nb_sync_helper(self, ovn_nb_synchronizer,
                                  networks, ports,
@@ -561,8 +583,9 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                                  add_subnet_dhcp_options_list,
                                  delete_dhcp_options_list,
                                  add_port_groups_list,
-                                 del_port_groups_list):
-        self._test_mocks_helper(ovn_nb_synchronizer)
+                                 del_port_groups_list,
+                                 test_logging=False):
+        self._test_mocks_helper(ovn_nb_synchronizer, test_logging)
 
         ovn_api = ovn_nb_synchronizer.ovn_api
         mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'from_worker').start()
@@ -694,8 +717,7 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
             create_router_port_calls,
             any_order=True)
 
-        self.assertEqual(len(del_router_list),
-                         ovn_api.delete_lrouter.call_count)
+        self.assertEqual(len(del_router_list), ovn_api.lr_del.call_count)
         update_router_port_calls = [mock.call(mock.ANY, p)
                                     for p in update_router_port_list]
         self.assertEqual(
@@ -705,10 +727,9 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
             update_router_port_calls,
             any_order=True)
 
-        delete_lrouter_calls = [mock.call(r['router'])
+        delete_lrouter_calls = [mock.call(r['router'], if_exists=True)
                                 for r in del_router_list]
-        ovn_api.delete_lrouter.assert_has_calls(
-            delete_lrouter_calls, any_order=True)
+        ovn_api.lr_del.assert_has_calls(delete_lrouter_calls, any_order=True)
 
         self.assertEqual(
             len(del_router_port_list),
@@ -737,7 +758,24 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
         ovn_api.delete_dhcp_options.assert_has_calls(
             delete_dhcp_options_calls, any_order=True)
 
-    def test_ovn_nb_sync_mode_repair(self):
+        if test_logging:
+            # 2 times when doing add_logging_options_to_acls and then
+            # 2 times because of the add_label_related used 2 times for the
+            # from-port and to-port drop acls
+            self.assertEqual(4, ovn_nb_synchronizer.ovn_log_driver.
+                             _pgs_from_log_obj.call_count)
+
+    def _fake_log_obj(self, **kwargs):
+        log_obj_defaults_dict = {
+            'uuid': uuidutils.generate_uuid(),
+            'resource_id': None,
+            'target_id': None,
+            'event': log_const.ALL_EVENT,
+        }
+        log_obj_obj_dict = {**log_obj_defaults_dict, **kwargs}
+        return mock.Mock(**log_obj_obj_dict)
+
+    def _test_ovn_nb_sync_mode_repair(self, test_logging=False):
 
         create_network_list = [{'net': {'id': 'n2', 'mtu': 1450},
                                 'ext_ids': {}}]
@@ -747,7 +785,7 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                          {'id': 'provnet-orphaned-segment',
                           'lswitch': 'neutron-n4'}]
         create_port_list = self.ports
-        for port in create_port_list:
+        for port in create_port_list.copy():
             if port['id'] in ['p1n1', 'fp1']:
                 # this will be skipped by the logic,
                 # because p1n1 is already in lswitch-port list
@@ -863,7 +901,14 @@ class TestOvnNbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                                       add_subnet_dhcp_options_list,
                                       delete_dhcp_options_list,
                                       add_port_groups_list,
-                                      del_port_groups_list)
+                                      del_port_groups_list,
+                                      test_logging)
+
+    def test_ovn_nb_sync_mode_repair(self):
+        self._test_ovn_nb_sync_mode_repair(test_logging=False)
+
+    def test_ovn_nb_sync_mode_repair_logs_created(self):
+        self._test_ovn_nb_sync_mode_repair(test_logging=True)
 
     def test_ovn_nb_sync_mode_log(self):
         create_network_list = []
@@ -1183,7 +1228,8 @@ class TestOvnSbSyncML2(test_mech_driver.OVNMechanismDriverTestCase):
                                return_value=hosts_in_neutron) as mock_ghmws:
             ovn_sb_synchronizer.sync_hostname_and_physical_networks(mock.ANY)
             mock_ghmws.assert_called_once_with(
-                mock.ANY, include_agent_types={ovn_const.OVN_CONTROLLER_AGENT})
+                mock.ANY,
+                include_agent_types=set(ovn_const.OVN_CONTROLLER_TYPES))
             all_hosts = set(hostname_with_physnets.keys()) | hosts_in_neutron
             self.assertEqual(
                 len(all_hosts),
